var canvasL		= document.getElementById("canvasLeft");
var ctxL		= canvasL.getContext("2d");
var canvasL_fg	= document.getElementById("canvasLeft_fg");
var ctxL_fg		= canvasL_fg.getContext("2d");

var canvasM		= document.getElementById("canvasMiddle");
var ctxM		= canvasM.getContext("2d");
var canvasM_fg	= document.getElementById("canvasMiddle_fg");
var ctxM_fg		= canvasM_fg.getContext("2d");

var canvasR		= document.getElementById("canvasRight");
var ctxR		= canvasR.getContext("2d");
var canvasR_fg	= document.getElementById("canvasRight_fg");
var ctxR_fg		= canvasR_fg.getContext("2d");

// these globals need to be 'objectified' into object so they can 
// be accessed in a modularized fasion, per canvas
var time_scale = 0.006; 		// 0.001
var ball;
var t0;
var dt;
var animId;
var force;
var acc;	
var g = 50;
var k = 0.01;
var rho = 1.5;
var V = 1;
var yLevel = canvasL.height/1.5;
var vfac = -0.8;
var radius = 20;

window.onload = init; 

function init() 
{
	// hackfest assignments
	canvas = canvasL;  // default to left canvas
	
	// create one ball
	ball = new Ball(radius,'#0000ff',1,0,true);
	ball.pos2D = new Vector2D(50,50);
	ball.velo2D = new Vector2D(20,-20); // right downward trajectory

	ball.draw(ctxL);
	ball.draw(ctxM);
	ball.draw(ctxR);
	
	// label each canvas
	ctxL_fg.font = "18px Comic Sans MS";
	ctxR_fg.font = "18px Comic Sans MS";	
	ctxM_fg.font = "18px Comic Sans MS";
	
	var align = 60;
		
	ctxL_fg.fillStyle = "red";
	ctxL_fg.textAlign = "center";
	ctxL_fg.fillText("Water", 0+canvasL.width/2, 30); 
	
	ctxM_fg.fillStyle = "green";
	ctxM_fg.textAlign = "center";
	ctxM_fg.fillText("Oil", 0+canvasM.width/2, 30); 
	
	ctxR_fg.fillStyle = "blue";
	ctxR_fg.textAlign = "center";
	ctxR_fg.fillText("Air", 0+canvasR.width/2, 30); 	
			
	// creates water line  // on a separate canvas (on top)
	ctxL_fg.fillStyle = "rgba(0,255,255,0.5)";
	ctxL_fg.fillRect(0,yLevel,canvasL_fg.width,canvasL_fg.height);
	
	ctxM_fg.fillStyle = "rgba(5,5,5,0.5)";
	ctxM_fg.fillRect(0,yLevel+10,canvasM_fg.width,canvasM_fg.height);
	
	ctxR_fg.fillStyle = "rgba(0,8,255,0.5)";
	ctxR_fg.fillRect(0,canvasR_fg.height-2,canvasL_fg.width,canvasL_fg.height);
	
	// set up event listeners
	addEventListener('mousedown',onDown,false);
	addEventListener('mouseup',onUp,false);	

	// initialise time and animate
	initAnim();	
}


function initAnim()
{
	t0 = new Date().getTime(); 
	animFrame();
}

function animFrame()
{
	animId = requestAnimationFrame(animFrame,canvas);
	onTimer(); 
}

function onTimer()
{
	var t1 = new Date().getTime(); 
	dt = time_scale *(t1-t0); 
	t0 = t1;
	if( dt>0.2 ) {dt=0;};	
	move();
}

function move()
{
	moveObject(ctxL, canvasL);
	moveObject(ctxR, canvasR);  
	moveObject(ctxM, canvasM);  
	
		// each ctx needs to have its own 'force field'
		
	calcForce();  	// combines gravity, upthrust, and drag
	updateAccel(); 	// considers mass of object
	updateVelo();
}

function stop()
{
	cancelAnimationFrame(animId);
}

function moveObject(context, canvas)
{
	ball.pos2D = ball.pos2D.addScaled(ball.velo2D,dt);	
	context.clearRect(0, 0, canvas.width, canvas.height);
	ball.draw(context);	
}

function onDown(evt) 
{
	ball.velo2D = new Vector2D(0,0); // keep ball still at mouse
					 // stop animation 
	ball.pos2D = new Vector2D(evt.clientX,evt.clientY-ball.radius);	
	moveObject(ctxL,canvasL);
	moveObject(ctxR,canvasR); // MH
	moveObject(ctxM,canvasM); // MH
	
	
	stop();
} 

function onUp(evt) 
{
	//ball.velo2D = new Vector2D(evt.clientX-ball.x,evt.clientY-ball.y);
	ball.velo2D = new Vector2D(20,-20); 
	initAnim();	// restart animation
} 

function calcForce()
{
	var gravity = Forces.constantGravity(ball.mass,g);
	var rball = ball.radius;
	var xball = ball.x;				
	var yball = ball.y;			
	var dr = (yball-yLevel)/rball;

	// calculate volume fraction of object that is submerged
	var ratio; 
	if (dr <= -1)
	{ // object (ball) is completely out of water
		ratio = 0;
	}
	else if (dr < 1)
	{ // object (ball) is partially in water 			
		ratio = 0.5 + 0.25*dr*(3-dr*dr); // for sphere
	}
	else
	{ // object is completely in water/submerged
		ratio = 1;
	}

	// physics / bouyancy
	var upthrust = new Vector2D(0,-rho*V*ratio*g);

	var drag = ball.velo2D.multiply(-ratio*k*ball.velo2D.length());

	//  combines the forces on the ball
	force = Forces.add([gravity, upthrust, drag]);	

	// check borders
	if (xball < rball)
	{
		ball.xpos = rball;
		ball.vx *= vfac;				
	}
	if (xball > canvas.width - rball)
	{
		ball.xpos = canvas.width - rball;
		ball.vx *= vfac;				
	}			
}	

function updateAccel()
{
	acc = force.multiply(1/ball.mass);
}

function updateVelo()
{
	ball.velo2D = ball.velo2D.addScaled(acc,dt);				
}

function drawLine(ctx,color)
{
    ctx.beginPath();
    ctx.moveTo(0,0);
    ctx.lineTo(ctx.canvas.width,ctx.canvas.height);
    ctx.strokeStyle=color;
    ctx.stroke();
}


