﻿var canvas = document.getElementById('canvas');
var context = canvas.getContext('2d'); 

var canvas_fg = document.getElementById('canvas_fg');
var context_fg = canvas_fg.getContext('2d'); 

var time_scale = 0.006; // 0.001
var ball;
var t0;
var dt;
var animId;
var force;
var acc;	
var g = 50;
var k = 0.01;
var rho = 1.5;
var V = 1;
var yLevel = canvas.height/1.5;
var vfac = -0.8;
var radius = 20;

window.onload = init; 

function init() 
{
	// create a ball
	ball = new Ball(radius,'#0000ff',1,0,true);
	ball.pos2D = new Vector2D(50,50);
	ball.velo2D = new Vector2D(20,-20); // right downward trajectory
	// ball.velo2D = new Vector2D(20,-60);	
	ball.draw(context);

	// create water // on a separate canvas (on top)
	context_fg.fillStyle = "rgba(0,255,255,0.5)";
	context_fg.fillRect(5,yLevel,canvas.width,canvas.height);

	// set up event listeners
	addEventListener('mousedown',onDown,false);
	addEventListener('mouseup',onUp,false);	

	// initialise time and animate
	initAnim();
};

function onDown(evt) 
{
	ball.velo2D = new Vector2D(0,0); // keep ball still at mouse
					 // stop animation 
	ball.pos2D = new Vector2D(evt.clientX,evt.clientY);	
	moveObject();
	stop();
} 

function onUp(evt) 
{
	//ball.velo2D = new Vector2D(evt.clientX-ball.x,evt.clientY-ball.y);
	ball.velo2D = new Vector2D(20,-20);
	initAnim();	// restart animation
} 

function initAnim()
{
	t0 = new Date().getTime(); 
	animFrame();
}

function animFrame()
{
	animId = requestAnimationFrame(animFrame,canvas);
	onTimer(); 
}

function onTimer()
{
	var t1 = new Date().getTime(); 
	dt = time_scale *(t1-t0); 
	t0 = t1;
	if( dt>0.2 ) {dt=0;};	
	move();
}
function move()
{
	moveObject();
	calcForce();  	// combines gravity, and water factor.
	updateAccel(); 	// considers mass of object
	updateVelo();
}
function stop()
{
	cancelAnimationFrame(animId);
}

// uses global canvas/and context
function moveObject()
{
	ball.pos2D = ball.pos2D.addScaled(ball.velo2D,dt);	
	context.clearRect(0, 0, canvas.width, canvas.height);
	ball.draw(context);	
}
function calcForce()
{
	var gravity = Forces.constantGravity(ball.mass,g);
	var rball = ball.radius;
	var xball = ball.x;				
	var yball = ball.y;			
	var dr = (yball-yLevel)/rball;


	// calculate volume fraction of object that is submerged
	var ratio; 
	if (dr <= -1)
	{ // object is completely out of water
		ratio = 0;
	}
	else if (dr < 1)
	{ // object is partially in water 			
		ratio = 0.5 + 0.25*dr*(3-dr*dr); // for sphere
	}
	else
	{ // object is completely in water/submerged
		ratio = 1;
	}


	// physics / bouyancy
	var upthrust = new Vector2D(0,-rho*V*ratio*g);

	var drag = ball.velo2D.multiply(-ratio*k*ball.velo2D.length());

	force = Forces.add([gravity, upthrust, drag]);	

	// check borders
	if (xball < rball)
	{
		ball.xpos = rball;
		ball.vx *= vfac;				
	}
	if (xball > canvas.width - rball)
	{
		ball.xpos = canvas.width - rball;
		ball.vx *= vfac;				
	}			
}	
function updateAccel()
{
	acc = force.multiply(1/ball.mass);
}	
function updateVelo()
{
	ball.velo2D = ball.velo2D.addScaled(acc,dt);				
}

